from zerohash import resources

OBJECT_CLASSES = {
    resources.DigitalAssetAddresses.OBJECT_NAME: resources.DigitalAssetAddresses,
    resources.LiquidityRfq.OBJECT_NAME: resources.LiquidityRfq,
    resources.ParticipantCustomerNew.OBJECT_NAME: resources.ParticipantCustomerNew,
    resources.LiquidityExecute.OBJECT_NAME: resources.LiquidityExecute,
    resources.TradesBatch.OBJECT_NAME: resources.TradesBatch,
}
